import logging
import shutil
from pathlib import Path

_logger = logging.getLogger(__name__)


def copy_children(source_dir: Path,
                  target_dir: Path,
                  merge_directories: bool = True,
                  overwrite_files: bool = False):
    """Low performance function for copying tree"""
    if not target_dir.exists():
        raise Exception(f"The target directory '{target_dir}' does not exist.")
    for child in source_dir.iterdir():
        target_path_for_child = target_dir.joinpath(child.name)
        if child.is_dir():
            if not merge_directories and target_path_for_child.exists():
                raise Exception(f"The directory (or possibly file) '{target_path_for_child}' does already exist.")
            _logger.debug(f"Copying directory {child} to {target_path_for_child}")
            target_path_for_child.mkdir(exist_ok=True)
            copy_children(child, target_path_for_child,
                          merge_directories=merge_directories,
                          overwrite_files=overwrite_files)
            # shutil.copytree(child, target_path_for_child, ) # not used because not configurable enough
        else:
            if target_path_for_child.exists():
                if not overwrite_files:
                    raise Exception(f"The file (or possibly directory) '{target_path_for_child}' does already exist. "
                                    f"It would be overwritten by {child}")
                else:
                    _logger.debug(f"Deleting {target_path_for_child} to overwrite it with {child}")
                    target_path_for_child.unlink()
            _logger.debug(f"Copying file {child} to {target_path_for_child}")
            shutil.copyfile(str(child), str(target_path_for_child))



import logging
import shutil
import tempfile
from pathlib import Path

_logger = logging.getLogger(__name__)


def create_temp_dir_with_manual_deletion_responsibility(preferred_name: str) -> Path:
    system_temp_dir = Path(tempfile.gettempdir())
    temp_dir = system_temp_dir.joinpath(preferred_name)
    if temp_dir.exists():
        temp_dir = Path(tempfile.mkdtemp(prefix=preferred_name))
    else:
        temp_dir.mkdir(exist_ok=False)
    return temp_dir


def create_temp_file_with_manual_deletion_responsibility(preferred_name: str) -> Path:
    system_temp_dir = Path(tempfile.gettempdir())
    temp_file = system_temp_dir.joinpath(preferred_name)
    if temp_file.exists():
        temporary_file_fd, temporary_file_name = tempfile.mkstemp(prefix=preferred_name)
        temp_file = Path(temporary_file_name)
    else:
        temp_file.touch(exist_ok=False)
    return temp_file


class CreateTempFileFor:
    def __init__(self,
                 file: Path,
                 cleanup: bool = True):
        self._file = file
        self._cleanup = cleanup

    def __enter__(self) -> Path:
        preferred_temp_file_name = self._file.name
        self._temp_file = create_temp_file_with_manual_deletion_responsibility(preferred_temp_file_name)
        if not self._file.exists():
            raise Exception(f"file {self._file} does not exist")
        self._temp_file.write_bytes(self._file.read_bytes())
        logging.info(f"{CreateTempFileFor.__name__}: Created temp file {str(self._temp_file)}")
        return self._temp_file

    def __exit__(self, exc_type, exc_value, traceback):
        if self._cleanup:
            logging.info(f"{CreateTempFileFor.__name__}: Removing temp file {str(self._temp_file)}")
            self._temp_file.unlink()


class CreateTempFile:
    def __init__(self,
                 preferred_name: str,
                 cleanup: bool = True):
        self._preferred_name = preferred_name
        self._cleanup = cleanup

    def __enter__(self) -> Path:
        self._temp_file = create_temp_file_with_manual_deletion_responsibility(self._preferred_name)
        logging.info(f"{CreateTempFileFor.__name__}: Created temp file {str(self._temp_file)}")
        return self._temp_file

    def __exit__(self, exc_type, exc_value, traceback):
        if self._cleanup:
            if self._temp_file.exists():
                logging.info(f"{CreateTempFileFor.__name__}: Removing temp file {str(self._temp_file)}")
                self._temp_file.unlink()


class CreateTempDirectory:
    def __init__(self,
                 preferred_name: str,
                 cleanup: bool = True):
        self._preferred_name = preferred_name
        self._cleanup = cleanup

    def __enter__(self) -> Path:
        self._temp_dir = create_temp_dir_with_manual_deletion_responsibility(self._preferred_name)
        logging.info(f"{CreateTempDirectory.__name__}: Created temp directory {str(self._temp_dir)}")
        return self._temp_dir

    def __exit__(self, exc_type, exc_value, traceback):
        if self._cleanup:
            if self._temp_dir.exists():
                logging.info(f"{CreateTempDirectory.__name__}: Removing temp directory {str(self._temp_dir)}")
                shutil.rmtree(self._temp_dir)


class CreateTempDirectoryFor:
    def __init__(self,
                 directory: Path,
                 cleanup: bool = True):
        self._directory = directory
        self._cleanup = cleanup

    def __enter__(self) -> Path:
        preferred_temp_dir_name = self._directory.name
        self._temp_directory = create_temp_dir_with_manual_deletion_responsibility(preferred_temp_dir_name)
        if not self._directory.exists():
            raise Exception(f"directory {self._directory} does not exist")
        self._temp_directory.rmdir()
        shutil.copytree(str(self._directory), str(self._temp_directory))
        logging.info(f"{CreateTempDirectoryFor.__name__}: Created temp directory {str(self._temp_directory)} "
                     f"as copy of {self._directory}")
        return self._temp_directory

    def __exit__(self, exc_type, exc_value, traceback):
        if self._cleanup:
            logging.info(f"{CreateTempDirectoryFor.__name__}: Removing temp directory {str(self._temp_directory)}")
            shutil.rmtree(self._temp_directory)



from pathlib import Path
from setuptools import find_packages, setup

readme = Path('README.rst').read_text(encoding="utf-8")
history = Path('docs/history.rst').read_text(encoding="utf-8")

runtime_requirements = []
development_requirements = [
    'pip>=19.0.2',
    'bumpversion>=0.5.3',
    'wheel>=0.32.3',
    'watchdog>=0.9.0',
    'flake8>=3.6.0',
    'tox>=3.6.1',
    'coverage>=4.5.2',
    'Sphinx>=1.8.3',
    'twine>=1.12.1',
    'pluggy>=0.7.0',
    'mypy>=0.650',
    'pytest>=3.8.2',
    'pytest-runner>=4.2',
    'pytest-mock>=1.10.1',
]

setup(
    author="Thomas Jahoda",
    author_email="thomasjahoda@users.noreply.github.com",
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="Utilities related to files",
    install_requires=runtime_requirements,
    extras_require={
        'dev': development_requirements
    },
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='tjpy_file_util',
    name='tjpy_file_util',
    packages=find_packages(include=['tjpy_file_util', 'tjpy_file_util.*']),
    test_suite='tests',
    url='https://github.com/thomasjahoda/tjpy_file_util',
    version='0.1.4',
    zip_safe=False,
)
